{-# LANGUAGE NamedFieldPuns #-}
module Conllu.Parse.Paragraph (
      Paragraph(..)
    , byParagraph
  ) where

import Conllu.Type (Doc, Sent(..))
import Data.List (find, isPrefixOf)

newtype Paragraph = Paragraph {
    sentences :: [Sent]
  }

-- | Assumes the document starts with a newpar (otherwise the first sentences
-- don't have a paragraph id to be associated to — this is always the case in
-- the output of our stanza annotator)
byParagraph :: Doc -> [Paragraph]
byParagraph = walk []
  where
    walk stack (s:ss) =
      case (getParagraph s, stack) of
        (Just _, []) -> walk [s] ss
        (Just _, _) -> Paragraph (reverse stack):(walk [s] ss)
        _ -> walk (s:stack) ss
    walk stack [] = [Paragraph $ reverse stack]

getParagraph :: Sent -> Maybe String
getParagraph = fmap snd . find (("newpar " `isPrefixOf`) . fst) . _meta
