{-# LANGUAGE OverloadedStrings #-}
module Data.BIO
  ( BIO(..)
  , Tree(..)
  , bio ) where

import Control.Applicative (empty)
import Data.ByteString as BS (splitAt)
import Data.Csv (FromField(..), ToField(..))
import Data.Foldable (toList)

data BIO a = Begin a | Inside a | Outside deriving (Show)

instance FromField a => FromField (BIO a) where
  parseField "O" = pure Outside
  parseField f =
    case BS.splitAt 2 f of
      ("B-", tag) -> Begin <$> parseField tag
      ("I-", tag) -> Inside <$> parseField tag
      _ -> empty

instance ToField a => ToField (BIO a) where
  toField Outside = "O"
  toField (Begin a) = "B-" <> toField a
  toField (Inside a) = "I-" <> toField a

data Tree a b = Leaf a | Node b [Tree a b] | Root [Tree a b] deriving (Show)

bio :: (Eq b, Foldable t) => (a -> BIO b) -> t a -> Tree a b
bio field = Root . atRoot . toList
  where
    atRoot [] = []
    atRoot (a:as) =
      case field a of
        Outside -> (Leaf a):(atRoot as)
        Begin b -> enter b a as
        Inside b -> enter b a as
    enter b a as =
      let (nodes, otherTrees) = inside [] b as in
      (Node b (Leaf a:nodes)):(atRoot otherTrees)
    inside stack b0 l =
      case field <$> take 1 l of
        [Inside b1]
          | b0 == b1 -> inside (Leaf (head l):stack) b0 (tail l)
        _ -> (reverse stack, l)
